==> repack-2.5.1.tgz: package/android/src/main/java/com/callstack/repack/RemoteChunkLoader.kt <==
package com.callstack.repack

import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactContext
import okhttp3.*
import java.io.File
import java.io.IOException
import java.io.OutputStreamWriter
import java.util.concurrent.TimeUnit

class RemoteChunkLoader(private val reactContext: ReactContext) {
    private val chunksDirName = "chunks"
    private val client = OkHttpClient()

    private fun getChunkFilePath(id: String): String {
        return "${chunksDirName}/$id.chunk.bundle"
    }

    private fun createClientPerRequest(config: ChunkConfig): OkHttpClient {
        val clientPerRequestBuilder = client.newBuilder();
        clientPerRequestBuilder.connectTimeout(config.timeout.toLong(), TimeUnit.MILLISECONDS);
        clientPerRequestBuilder.readTimeout(config.timeout.toLong(), TimeUnit.MILLISECONDS)

        return clientPerRequestBuilder.build()
    }

    private fun downloadAndCache(config: ChunkConfig, onSuccess: () -> Unit, onError: (code: String, message: String) -> Unit) {
        val path = getChunkFilePath(config.id)
        val file = File(reactContext.filesDir, path)

        val callback = object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                onError(
                        ChunkLoadingError.NetworkFailure.code,
                        e.message ?: e.toString()
                )
            }

            override fun onResponse(call: Call, response: Response) {
                if (response.isSuccessful) {
                    try {
                        val chunksDir = File(reactContext.filesDir, chunksDirName)
                        if (!chunksDir.exists()) {
                            File(reactContext.filesDir, chunksDirName).mkdir()
                        }

                        file.createNewFile()

                        val body = response.body?.string()
                        val outputStream = file.outputStream()
                        val writer = OutputStreamWriter(outputStream)
                        writer.write(body)
                        writer.close()
                        onSuccess()
                    } catch (error: Exception) {
                        onError(
                                ChunkLoadingError.ChunkCachingFailure.code,
                                error.message ?: error.toString()
                        )
                    }
                } else {
                    onError(
                            ChunkLoadingError.RequestFailure.code,
                            "Request should have returned with 200 HTTP status, but instead it received ${response.code}"
                    )
                }
            }
        }

        val clientPerRequest = createClientPerRequest(config)
        var request = Request.Builder()
                .url(config.url)
                .headers(config.headers)


        if (config.method == "POST" && config.body != null) {
            request = request.post(config.body)
        }

        clientPerRequest.newCall(request.build()).enqueue(callback)
    }

    fun execute(config: ChunkConfig, promise: Promise) {
        try {
            val path = getChunkFilePath(config.id)
            reactContext.catalystInstance.loadScriptFromFile(
                    "${reactContext.filesDir}/${path}",
                    config.url.toString(),
                    false
            )
            promise.resolve(null)
        } catch (error: Exception) {
            promise.reject(
                    ChunkLoadingError.RemoteEvalFailure.code,
                    error.message ?: error.toString()
            )
        }
    }


    fun preload(config: ChunkConfig, promise: Promise) {
        downloadAndCache(config, { promise.resolve(null) }, { code, message -> promise.reject(code, message) })
    }

    fun load(config: ChunkConfig, promise: Promise) {
        downloadAndCache(config, {
            execute(config, promise)
        }, { code, message -> promise.reject(code, message) })
    }

    fun invalidate(chunkId: String?) {
        if (chunkId != null) {
            val file = File(reactContext.filesDir, getChunkFilePath(chunkId))

            if (file.exists()) {
                file.delete()
            }
        }
    }

    fun invalidateAll() {
        val file = File(reactContext.filesDir, chunksDirName)
        if (file.exists()) {
            file.deleteRecursively()
        }
    }
}

==> repack-3.2.0-rc.0.tgz: package/android/src/main/java/com/callstack/repack/RemoteChunkLoader.kt <==
package com.callstack.repack

import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactContext
import okhttp3.*
import java.io.File
import java.io.IOException
import java.io.OutputStreamWriter
import java.util.concurrent.TimeUnit

class RemoteScriptLoader(private val reactContext: ReactContext) {
    private val scriptsDirName = "scripts"
    private val client = OkHttpClient()

    private fun getScriptFilePath(id: String): String {
        return "${scriptsDirName}/$id.script.bundle"
    }

    private fun createClientPerRequest(config: ScriptConfig): OkHttpClient {
        val clientPerRequestBuilder = client.newBuilder();
        clientPerRequestBuilder.connectTimeout(config.timeout.toLong(), TimeUnit.MILLISECONDS);
        clientPerRequestBuilder.readTimeout(config.timeout.toLong(), TimeUnit.MILLISECONDS)

        return clientPerRequestBuilder.build()
    }

    private fun downloadAndCache(config: ScriptConfig, onSuccess: () -> Unit, onError: (code: String, message: String) -> Unit) {
        val path = getScriptFilePath(config.id)
        val file = File(reactContext.filesDir, path)

        val callback = object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                onError(
                        ScriptLoadingError.NetworkFailure.code,
                        e.message ?: e.toString()
                )
            }

            override fun onResponse(call: Call, response: Response) {
                if (response.isSuccessful) {
                    try {
                        val scriptsDir = File(reactContext.filesDir, scriptsDirName)
                        if (!scriptsDir.exists()) {
                            File(reactContext.filesDir, scriptsDirName).mkdir()
                        }

                        val body = response.body?.string()

                        if (config.verifyScriptSignature == true) {
                            CodeSigningUtils.verifyBundle(reactContext, config.token, body)
                        }

                        file.createNewFile()

                        val outputStream = file.outputStream()
                        val writer = OutputStreamWriter(outputStream)
                        writer.write(body)
                        writer.close()
                        onSuccess()
                    } catch (error: Exception) {
                        onError(
                                ScriptLoadingError.ScriptCachingFailure.code,
                                error.message ?: error.toString()
                        )
                    }
                } else {
                    onError(
                            ScriptLoadingError.RequestFailure.code,
                            "Request should have returned with 200 HTTP status, but instead it received ${response.code}"
                    )
                }
            }
        }

        val clientPerRequest = createClientPerRequest(config)
        var request = Request.Builder()
                .url(config.url)
                .headers(config.headers)


        if (config.method == "POST" && config.body != null) {
            request = request.post(config.body)
        }

        clientPerRequest.newCall(request.build()).enqueue(callback)
    }

    fun execute(config: ScriptConfig, promise: Promise) {
        try {
            val path = getScriptFilePath(config.id)
            reactContext.catalystInstance.loadScriptFromFile(
                    "${reactContext.filesDir}/${path}",
                    config.url.toString(),
                    false
            )
            promise.resolve(null)
        } catch (error: Exception) {
            promise.reject(
                    ScriptLoadingError.RemoteEvalFailure.code,
                    error.message ?: error.toString()
            )
        }
    }


    fun prefetch(config: ScriptConfig, promise: Promise) {
        downloadAndCache(config, { promise.resolve(null) }, { code, message -> promise.reject(code, message) })
    }

    fun load(config: ScriptConfig, promise: Promise) {
        downloadAndCache(config, {
            execute(config, promise)
        }, { code, message -> promise.reject(code, message) })
    }

    fun invalidate(scriptId: String?) {
        if (scriptId != null) {
            val file = File(reactContext.filesDir, getScriptFilePath(scriptId))

            if (file.exists()) {
                file.delete()
            }
        }
    }

    fun invalidateAll() {
        val file = File(reactContext.filesDir, scriptsDirName)
        if (file.exists()) {
            file.deleteRecursively()
        }
    }
}
